export { PDFScriptingManagerComponents as PDFScriptingManager };
declare class PDFScriptingManagerComponents extends PDFScriptingManager {
    constructor(options: any);
}
import { PDFScriptingManager } from "./pdf_scripting_manager.js";
